﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using CsvHelper.Configuration.Attributes;
using Microsoft.AspNetCore;
using Microsoft.AspNetCore.Hosting;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Storage;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using TinyCsvParser.Mapping;

namespace api_core_v2
{
  public class Program
  {
    public static void Main(string[] args)
    {
      CreateWebHostBuilder(args).Build().Run();
    }

    public static IWebHostBuilder CreateWebHostBuilder(string[] args) =>
        WebHost.CreateDefaultBuilder(args)
            .UseStartup<Startup>();
  }
}

public class IpRangeItem
{
  // public int Id { get; set; }

  [Index(0)]
  public long RangeMin { get; set; }
  
  [Index(1)]
  public long RangeMax { get; set; }

  [Index(2)]
  public string Code { get; set; }
  // public string Name { get; set; }

  public override string ToString()
  {
    return $"{this.IPint2dotted(this.RangeMin)}-{this.IPint2dotted(this.RangeMax)} {this.Code}";
  }

  public string IPint2dotted(long ip)
  {
    string ret = "";
    try
    {
      string hex = ip.ToString("X");


      if (hex.Length >= 8)
        hex = hex.Substring(hex.Length - 8);
      else
        hex = hex.PadLeft(8, '0');


      for (int i = hex.Length; i > 0; i -= 2)
      {
        ret += long.Parse(Convert.ToString(hex[i - 2].ToString() + hex[i - 1].ToString()), System.Globalization.NumberStyles.HexNumber).ToString() + ".";
      }


      if (ret.Length > 0)
        return ret.Substring(0, ret.Length - 1);
      else
        return "";
    }
    catch (Exception exc)
    {
      return "";
    }


  }
}

public class IpRangeCsvMapping : CsvMapping<IpRangeItem>
{
  public IpRangeCsvMapping() : base()
  {
    MapProperty(0, x => x.RangeMin);
    MapProperty(1, x => x.RangeMax);
    MapProperty(2, x => x.Code);
    // MapProperty(3, x => x.Name);
  }
}

public class IpRangeContext : DbContext
{
  public DbSet<IpRangeItem> IpRanges { get; set; }

  protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
  {
    optionsBuilder.UseInMemoryDatabase();
  }
}