﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using TinyCsvParser;

namespace api_core_v2
{
  public class Startup
  {
    public Startup(IConfiguration configuration)
    {
      Configuration = configuration;
    }

    public IConfiguration Configuration { get; }

    // This method gets called by the runtime. Use this method to add services to the container.
    public void ConfigureServices(IServiceCollection services)
    {
      services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_2);
      services.AddDbContext<IpRangeContext>();

      services.AddSingleton(new SortedList<long, IpRangeItem>(235000));
    }

    // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
    public void Configure(IApplicationBuilder app, IHostingEnvironment env, IServiceProvider provider)
    {
      if (env.IsDevelopment())
      {
        app.UseDeveloperExceptionPage();
      }
      else
      {
        // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
        app.UseHsts();
      }

      app.UseHttpsRedirection();
      app.UseMvc();

      // Import_v1(provider);
      Import_v2(provider);
    }

    private void Import_v2(IServiceProvider provider)
    {
      using (var reader = new StreamReader("sample.csv"))
      using (var csv = new CsvHelper.CsvReader(reader))
      {
        csv.Configuration.HasHeaderRecord = false;
        var records = csv.GetRecords<IpRangeItem>();
        // records.ToList().ForEach(Console.WriteLine);

        var sortedList = provider.GetService<SortedList<long, IpRangeItem>>();

        try
        {
          records.ToList().ForEach(x =>
          {
            try
            {
              sortedList.Add(x.RangeMax, x);
            }
            catch (Exception e)
            {
              throw;
            }

          });
        }
        catch (Exception ex)
        {
          throw;
        }
      }
    }

    void Import_v1(IServiceProvider _)
    {
      var csvParser = new CsvParser<IpRangeItem>(new CsvParserOptions(false, ','), new IpRangeCsvMapping());

      // var csvPath = Path.GetFullPath( Path.Combine("..", "importer", "IP2LOCATION-LITE-DB1.CSV"));
      // var csvPath = "IP2LOCATION-LITE-DB1.CSV";
      var csvPath = "sample.csv";

      var csvResult = csvParser
          .ReadFromFile(csvPath, System.Text.Encoding.UTF8);

      var result = csvResult.Select(x => x.Result)
          .Where(x => x != null && x.Code != "-")
          .ToArray();

      // #if DEBUG
      //       result.ToList().ForEach(r=>{
      //               System.Console.WriteLine($"{r.RangeMin} {r.RangeMax} {r.Code} {r.Name}");
      //           });
      // #endif          

      var db = _.GetService<IpRangeContext>();
      var sortedList = _.GetService<SortedList<long, IpRangeItem>>();

      result.ToList().ForEach(x =>
      {
        sortedList.Add(x.RangeMin, x);
      });

      // db.IpRanges.AddRange(result.AsEnumerable());

      // db.SaveChanges();
    }
  }
}
