#!/bin/env python
import redis
import csv
from pprint import pprint as print
from collections import namedtuple
import socket
import struct
import re
import json
import os

RedisItem = namedtuple('RedisItem', field_names=[
                       'rangeMin', 'rangeMax', 'countryCode', 'countryName'])



REDIS_HOST = os.environ.get('REDIS_HOST', 'localhost')
REDIS_PORT = os.environ.get('REDIS_PORT', 6379)
REDIS_INDEX = os.environ.get('REDIS_INDEX', 'iprange:locations')

REDIS_LIGHT_INDEX = os.environ.get('REDIS_INDEX', 'iprange:locations:light')

IP2LOCATION_DATAFILE = os.environ.get('IP2LOCATION_DATAFILE', 'IP2LOCATION-LITE-DB1.CSV')


client = redis.Redis(host=REDIS_HOST, port=REDIS_PORT, db=0)


def ip2long(ip_IT):
    """
    Convert an IP string to long
    """
    ip_packed = socket.inet_aton(ip_IT)
    return struct.unpack("!L", ip_packed)[0]

def get(ip_IT, redisConn):
    """
    Get a range and all its data by ip_IT
    """
    
    ipnum = ip2long(ip_IT)

    #get the location record from redis
    record = redisConn.zrangebyscore(REDIS_INDEX, ipnum ,'+inf', 0, 1, True)
    if not record:
        #not found? k!
        return None

    #extract location id
    try:
        data = json.loads(record[0][0].decode('utf-8'))
        return data
    except IndexError:
        return None

    #address not in any range
    if not rngMin <= ipnum <= rngMax:
        return None
    
    return "found: %s" % record

    #IPRange(rngMin, rngMax, lat, lon, zipcode)

def get_countrycode(ip, redisConn):
    """
    Get a range and all its data by ip
    """
    
    ipnum = ip2long(ip)

    #get the location record from redis
    record = redisConn.zrangebyscore(REDIS_INDEX, ipnum ,'+inf', 0, 1, True)
    if not record:
        #not found? k!
        return None

    #extract location id
    try:
        data = record[0][0].decode('utf-8')
        return data
    except IndexError:
        return None

    #address not in any range
    if not rngMin <= ipnum <= rngMax:
        return None
    
    return "found: %s" % record

    #IPRange(rngMin, rngMax, lat, lon, zipcode)


def import_data(filename):
    # client.flushdb(0)

    pipe = client.pipeline()

    with open(IP2LOCATION_DATAFILE, newline='') as csvfile:
        reader = csv.reader(csvfile, delimiter=',', quotechar='"')
        for row in reader:
            item = RedisItem(
                rangeMin=int(row[0]),
                rangeMax=int(row[1]),
                countryCode=row[2],
                countryName=row[3]
            )

            # print(item._asdict())

            if(item.countryCode == '-'):
                continue

            mapping  =  dict()
            mapping[json.dumps(item._asdict())] = item.rangeMax

            pipe.zadd(REDIS_INDEX, mapping)

    pipe.execute()


def import_data_light(filename):
    client.flushdb(0)    

    pipe = client.pipeline()

    with open(IP2LOCATION_DATAFILE, newline='') as csvfile:
        reader = csv.reader(csvfile, delimiter=',', quotechar='"')
        for row in reader:
            countryCode = row[2]
            rangeMax = row[1]
            # print(item._asdict())

            if(countryCode == '-'):
                continue

            mapping  =  dict()
            mapping[countryCode] = rangeMax

            pipe.zadd(REDIS_LIGHT_INDEX, mapping)

    pipe.execute()

if __name__ == "__main__":
    ip_IT = '2.16.87.255'
    ip_TR = '213.14.95.132'

    # import_data(IP2LOCATION_DATAFILE)
    # italy = get(ip_IT, client)
    # turkey =get(ip_TR, client)
    # print(ip2long(ip_IT))
    # print(italy['countryCode'])

    # print(ip2long(ip_TR))
    # print(turkey['countryCode'])

    import_data_light(IP2LOCATION_DATAFILE)
    italy = get_countrycode(ip_IT, client)
    turkey =get_countrycode(ip_TR, client)
    print(italy)
    print(turkey)